@@ -25,10 +25,14 @@
  */
 
 #include "archive_kerfuffle.h"
+#include "pluginmanager.h"
 #include "jobs.h"
 #include "testhelper.h"
 
+#include <KIO/Global>
+
 #include <QDirIterator>
+#include <QMimeDatabase>
 #include <QStandardPaths>
 #include <QTest>
 
@@ -41,6 +45,11 @@ class ExtractTest : public QObject
 private Q_SLOTS:
     void testExtraction_data();
     void testExtraction();
+    void testPreservePermissions_data();
+    void testPreservePermissions();
+
+private:
+    PluginManager m_pluginManager;
 };
 
 QTEST_GUILESS_MAIN(ExtractTest)
@@ -269,22 +278,25 @@ void ExtractTest::testExtraction_data()
             << optionsPreservePaths
             << 7;
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.lzo");
-    QTest::newRow("extract selected entries from a lzop-compressed tarball without path")
-            << archivePath
-            << QVector<Archive::Entry*> {
-                   new Archive::Entry(this, QStringLiteral("file3.txt"), QString()),
-                   new Archive::Entry(this, QStringLiteral("dir2/file22.txt"), QString())
-               }
-            << optionsNoPaths
-            << 2;
+    // Only run tests if tar.lzo format is available
+    if (PluginManager().supportedMimeTypes().contains(QStringLiteral("application/x-tzo"))) {
+        archivePath = QFINDTESTDATA("data/simplearchive.tar.lzo");
+        QTest::newRow("extract selected entries from a lzop-compressed tarball without path")
+                << archivePath
+                << QVector<Archive::Entry*> {
+                       new Archive::Entry(this, QStringLiteral("file3.txt"), QString()),
+                       new Archive::Entry(this, QStringLiteral("dir2/file22.txt"), QString())
+                   }
+                << optionsNoPaths
+                << 2;
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.lzo");
-    QTest::newRow("extract all entries from a lzop-compressed tarball with path")
-            << archivePath
-            << QVector<Archive::Entry*>()
-            << optionsPreservePaths
-            << 7;
+        archivePath = QFINDTESTDATA("data/simplearchive.tar.lzo");
+        QTest::newRow("extract all entries from a lzop-compressed tarball with path")
+                << archivePath
+                << QVector<Archive::Entry*>()
+                << optionsPreservePaths
+                << 7;
+    }
 
     // Only run test for lrzipped tar if lrzip executable is found in path.
     if (!QStandardPaths::findExecutable(QStringLiteral("lrzip")).isEmpty()) {
@@ -441,4 +453,68 @@ void ExtractTest::testExtraction()
     archive->deleteLater();
 }
 
+void ExtractTest::testPreservePermissions_data()
+{
+    QTest::addColumn<QString>("archiveName");
+    QTest::addColumn<Plugin*>("plugin");
+    QTest::addColumn<QString>("testFile");
+    QTest::addColumn<int>("expectedPermissions");
+
+    // Repeat the same test case for each format and for each plugin supporting the format.
+    foreach (const QString &format, TestHelper::testFormats()) {
+        const QString filename = QFINDTESTDATA(QStringLiteral("data/test_permissions.%1").arg(format));
+        const auto mime = QMimeDatabase().mimeTypeForFile(filename, QMimeDatabase::MatchExtension);
+        const auto plugins = m_pluginManager.preferredWritePluginsFor(mime);
+        foreach (const auto plugin, plugins) {
+            QTest::newRow(QStringLiteral("test preserve 0755 permissions (%1, %2)").arg(format, plugin->metaData().pluginId()).toUtf8())
+                << filename
+                << plugin
+                << QStringLiteral("0755.sh")
+                << 0755;
+        }
+    }
+}
+
+void ExtractTest::testPreservePermissions()
+{
+    QFETCH(QString, archiveName);
+    QFETCH(Plugin*, plugin);
+    QVERIFY(plugin);
+    auto loadJob = Archive::load(archiveName, plugin);
+    QVERIFY(loadJob);
+    loadJob->setAutoDelete(false);
+
+    TestHelper::startAndWaitForResult(loadJob);
+    auto archive = loadJob->archive();
+    QVERIFY(archive);
+
+    if (!archive->isValid()) {
+        QSKIP("Could not find a plugin to handle the archive. Skipping test.", SkipSingle);
+    }
+
+    QTemporaryDir destDir;
+    if (!destDir.isValid()) {
+        QSKIP("Could not create a temporary directory for extraction. Skipping test.", SkipSingle);
+    }
+
+    auto extractionJob = archive->extractFiles({}, destDir.path());
+    QVERIFY(extractionJob);
+    extractionJob->setAutoDelete(false);
+    TestHelper::startAndWaitForResult(extractionJob);
+
+    // Check whether extraction preserved the original permissions.
+    QFETCH(QString, testFile);
+    QFile file(QStringLiteral("%1/%2").arg(destDir.path(), testFile));
+    QVERIFY(file.exists());
+    QFETCH(int, expectedPermissions);
+    const auto expectedQtPermissions = KIO::convertPermissions(expectedPermissions);
+    // On Linux we get also the XXXUser flags which are ignored by KIO::convertPermissions(),
+    // so we need to remove them before the comparison with the expected permissions.
+    QCOMPARE(file.permissions() & expectedQtPermissions, expectedQtPermissions);
+
+    loadJob->deleteLater();
+    extractionJob->deleteLater();
+    archive->deleteLater();
+}
+
 #include "extracttest.moc"
