@@ -39,11 +39,15 @@ class KERFUFFLE_EXPORT SettingsPage : public QWidget
     Q_OBJECT
 
 public:
-    explicit SettingsPage(QWidget *parent = 0, const QString &name = QString(), const QString &iconName = QString());
+    explicit SettingsPage(QWidget *parent = nullptr, const QString &name = QString(), const QString &iconName = QString());
 
     QString name() const;
     QString iconName() const;
 
+public slots:
+    virtual void slotSettingsChanged();
+    virtual void slotDefaultsButtonClicked();
+
 private:
     QString m_name;
     QString m_iconName;
