@@ -23,10 +23,11 @@
  */
 
 #include "archivemodel.h"
+#include "ark_debug.h"
 #include "jobs.h"
 
+#include <KIO/Global>
 #include <KLocalizedString>
-#include <kio/global.h>
 
 #include <QDBusConnection>
 #include <QMimeData>
@@ -37,29 +38,9 @@
 using namespace Kerfuffle;
 
 // Used to speed up the loading of large archives.
-static Archive::Entry *s_previousMatch = Q_NULLPTR;
+static Archive::Entry *s_previousMatch = nullptr;
 Q_GLOBAL_STATIC(QStringList, s_previousPieces)
 
-/**
- * Meta data related to one entry in a compressed archive.
- *
- * This is used for indexing entry properties as numbers
- * and for determining data displaying order in part's view.
- */
-enum EntryMetaDataType {
-    FullPath,            /**< The entry's file name */
-    Size,                /**< The entry's original size */
-    CompressedSize,      /**< The compressed size for the entry */
-    Permissions,         /**< The entry's permissions */
-    Owner,               /**< The user the entry belongs to */
-    Group,               /**< The user group the entry belongs to */
-    Ratio,               /**< The compression ratio for the entry */
-    CRC,                 /**< The entry's CRC */
-    Method,              /**< The compression method used on the entry */
-    Version,             /**< The archiver version needed to extract the entry */
-    Timestamp            /**< The timestamp for the current entry */
-};
-
 ArchiveModel::ArchiveModel(const QString &dbusPathName, QObject *parent)
     : QAbstractItemModel(parent)
     , m_dbusPathName(dbusPathName)
@@ -101,10 +82,10 @@ QVariant ArchiveModel::data(const QModelIndex &index, int role) const
                 return entry->name();
             case Size:
                 if (entry->isDir()) {
-                    int dirs;
-                    int files;
-                    const int children = childCount(index, dirs, files);
-                    return KIO::itemsSummaryString(children, files, dirs, 0, false);
+                    uint dirs;
+                    uint files;
+                    entry->countChildren(dirs, files);
+                    return KIO::itemsSummaryString(dirs + files, files, dirs, 0, false);
                 } else if (!entry->property("link").toString().isEmpty()) {
                     return QVariant();
                 } else {
@@ -171,7 +152,7 @@ Qt::ItemFlags ArchiveModel::flags(const QModelIndex &index) const
         return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | defaultFlags;
     }
 
-    return 0;
+    return nullptr;
 }
 
 QVariant ArchiveModel::headerData(int section, Qt::Orientation, int role) const
@@ -225,8 +206,8 @@ QModelIndex ArchiveModel::index(int row, int column, const QModelIndex &parent)
 
         Q_ASSERT(parentEntry->isDir());
 
-        const Archive::Entry *item = parentEntry->entries().value(row, Q_NULLPTR);
-        if (item != Q_NULLPTR) {
+        const Archive::Entry *item = parentEntry->entries().value(row, nullptr);
+        if (item != nullptr) {
             return createIndex(row, column, const_cast<Archive::Entry*>(item));
         }
     }
@@ -253,29 +234,7 @@ Archive::Entry *ArchiveModel::entryForIndex(const QModelIndex &index)
         Q_ASSERT(item);
         return item;
     }
-    return Q_NULLPTR;
-}
-
-int ArchiveModel::childCount(const QModelIndex &index, int &dirs, int &files) const
-{
-    if (index.isValid()) {
-        dirs = files = 0;
-        Archive::Entry *item = static_cast<Archive::Entry*>(index.internalPointer());
-        Q_ASSERT(item);
-        if (item->isDir()) {
-            const QVector<Archive::Entry*> entries = item->entries();
-            foreach(const Archive::Entry *entry, entries) {
-                if (entry->isDir()) {
-                    dirs++;
-                } else {
-                    files++;
-                }
-            }
-            return entries.count();
-        }
-        return 0;
-    }
-    return -1;
+    return nullptr;
 }
 
 int ArchiveModel::rowCount(const QModelIndex &parent) const
@@ -298,80 +257,6 @@ int ArchiveModel::columnCount(const QModelIndex &parent) const
     return m_showColumns.size();
 }
 
-void ArchiveModel::sort(int column, Qt::SortOrder order)
-{
-    if (m_showColumns.size() <= column) {
-        return;
-    }
-
-    emit layoutAboutToBeChanged();
-
-    QVector<Archive::Entry*> dirEntries;
-    m_rootEntry->returnDirEntries(&dirEntries);
-    dirEntries.append(m_rootEntry.data());
-
-    foreach(Archive::Entry *dir, dirEntries) {
-        QVector < QPair<Archive::Entry*,int> > sorting(dir->entries().count());
-        for (int i = 0; i < dir->entries().count(); ++i) {
-            Archive::Entry *item = dir->entries().at(i);
-            sorting[i].first = item;
-            sorting[i].second = i;
-        }
-
-        std::stable_sort(sorting.begin(), sorting.end(), [=](const QPair<Archive::Entry*, int> &left, const QPair<Archive::Entry*, int> &right) {
-            const auto leftEntry = left.first;
-            const auto rightEntry = right.first;
-            bool isLessThan = false;  // Whether the left entry is less than the right entry.
-
-            // #234373: sort folders before files.
-            if ((leftEntry->isDir()) && (!rightEntry->isDir())) {
-                isLessThan = true;
-            } else if ((!leftEntry->isDir()) && (rightEntry->isDir())) {
-                isLessThan = false;
-            } else {
-                const QVariant leftEntryMetaData = leftEntry->property(m_propertiesMap[column]);
-                const QVariant rightEntryMetaData = rightEntry->property(m_propertiesMap[column]);
-
-                switch (column) {
-                case FullPath:
-                    isLessThan = leftEntry->name() < rightEntry->name();
-                    break;
-                case Size:
-                case CompressedSize:
-                    isLessThan = leftEntryMetaData.toInt() < rightEntryMetaData.toInt();
-                    break;
-                default:
-                    isLessThan = leftEntryMetaData.toString() < rightEntryMetaData.toString();
-                    break;
-                }
-            }
-
-            if (order == Qt::AscendingOrder) {
-                return isLessThan;
-            }
-            // Descending order.
-            return !isLessThan;
-        });
-
-        QModelIndexList fromIndexes;
-        QModelIndexList toIndexes;
-        for (int r = 0; r < sorting.count(); ++r) {
-            Archive::Entry *item = sorting.at(r).first;
-            toIndexes.append(createIndex(r, 0, item));
-            fromIndexes.append(createIndex(sorting.at(r).second, 0, sorting.at(r).first));
-            dir->setEntryAt(r, sorting.at(r).first);
-        }
-
-        changePersistentIndexList(fromIndexes, toIndexes);
-
-        emit dataChanged(
-            index(0, 0, indexForEntry(dir)),
-            index(dir->entries().size() - 1, 0, indexForEntry(dir)));
-    }
-
-    emit layoutChanged();
-}
-
 Qt::DropActions ArchiveModel::supportedDropActions() const
 {
     return Qt::CopyAction | Qt::MoveAction;
@@ -426,7 +311,7 @@ bool ArchiveModel::dropMimeData(const QMimeData *data, Qt::DropAction action, in
         paths << url.toLocalFile();
     }
 
-    const Archive::Entry *entry = Q_NULLPTR;
+    const Archive::Entry *entry = nullptr;
     QModelIndex droppedOnto = index(row, column, parent);
     if (droppedOnto.isValid()) {
         entry = entryForIndex(droppedOnto);
@@ -467,7 +352,7 @@ Archive::Entry *ArchiveModel::parentFor(const Archive::Entry *entry, InsertBehav
 {
     QStringList pieces = entry->fullPath().split(QLatin1Char('/'), QString::SkipEmptyParts);
     if (pieces.isEmpty()) {
-        return Q_NULLPTR;
+        return nullptr;
     }
     pieces.removeLast();
 
@@ -694,8 +579,8 @@ Kerfuffle::Archive* ArchiveModel::archive() const
 
 void ArchiveModel::reset()
 {
-    m_archive.reset(Q_NULLPTR);
-    s_previousMatch = Q_NULLPTR;
+    m_archive.reset(nullptr);
+    s_previousMatch = nullptr;
     s_previousPieces->clear();
     initRootEntry();
 
@@ -766,7 +651,7 @@ OpenWithJob *ArchiveModel::openWith(Archive::Entry *file) const
 AddJob* ArchiveModel::addFiles(QVector<Archive::Entry*> &entries, const Archive::Entry *destination, const CompressionOptions& options)
 {
     if (!m_archive) {
-        return Q_NULLPTR;
+        return nullptr;
     }
 
     if (!m_archive->isReadOnly()) {
@@ -777,13 +662,13 @@ AddJob* ArchiveModel::addFiles(QVector<Archive::Entry*> &entries, const Archive:
 
         return job;
     }
-    return Q_NULLPTR;
+    return nullptr;
 }
 
 Kerfuffle::MoveJob *ArchiveModel::moveFiles(QVector<Archive::Entry*> &entries, Archive::Entry *destination, const CompressionOptions &options)
 {
     if (!m_archive) {
-        return Q_NULLPTR;
+        return nullptr;
     }
 
     if (!m_archive->isReadOnly()) {
@@ -796,12 +681,12 @@ Kerfuffle::MoveJob *ArchiveModel::moveFiles(QVector<Archive::Entry*> &entries, A
 
         return job;
     }
-    return Q_NULLPTR;
+    return nullptr;
 }
 Kerfuffle::CopyJob *ArchiveModel::copyFiles(QVector<Archive::Entry*> &entries, Archive::Entry *destination, const CompressionOptions &options)
 {
     if (!m_archive) {
-        return Q_NULLPTR;
+        return nullptr;
     }
 
     if (!m_archive->isReadOnly()) {
@@ -812,7 +697,7 @@ Kerfuffle::CopyJob *ArchiveModel::copyFiles(QVector<Archive::Entry*> &entries, A
 
         return job;
     }
-    return Q_NULLPTR;
+    return nullptr;
 }
 
 DeleteJob* ArchiveModel::deleteFiles(QVector<Archive::Entry*> entries)
@@ -827,7 +712,7 @@ DeleteJob* ArchiveModel::deleteFiles(QVector<Archive::Entry*> entries)
         connect(job, &DeleteJob::userQuery, this, &ArchiveModel::slotUserQuery);
         return job;
     }
-    return Q_NULLPTR;
+    return nullptr;
 }
 
 void ArchiveModel::encryptArchive(const QString &password, bool encryptHeader)
@@ -871,7 +756,7 @@ bool ArchiveModel::conflictingEntries(QList<const Archive::Entry*> &conflictingE
         bool isDir = entry.right(1) == QLatin1String("/");
         const Archive::Entry *archiveEntry = lastDirEntry->find(entry.split(QLatin1Char('/'), QString::SkipEmptyParts).last());
 
-        if (archiveEntry != Q_NULLPTR) {
+        if (archiveEntry != nullptr) {
             if (archiveEntry->isDir() != isDir || !allowMerging) {
                 if (isDir) {
                     skippedDirPath = lastDirEntry->fullPath();
@@ -1008,3 +893,13 @@ qulonglong ArchiveModel::uncompressedSize() const
 {
     return m_uncompressedSize;
 }
+
+QList<int> ArchiveModel::shownColumns() const
+{
+    return m_showColumns;
+}
+
+QMap<int, QByteArray> ArchiveModel::propertiesMap() const
+{
+    return m_propertiesMap;
+}
